MODULE LrgCards;

FROM SYSTEM IMPORT BYTE;
IMPORT Types, LC, Config;
IMPORT IO, Lib;
IMPORT WinBase, Tmr0;


CONST
  Now ::= WinBase.GetTickCount;

PROCEDURE Fatal( s-: ARRAY OF CHAR );
BEGIN
  Lib.FatalError( s );
  HALT;
END Fatal;


PROCEDURE Test01();
VAR
  s: ARRAY [0..31] OF CHAR;
  ok: BOOLEAN;
  x: CARDINAL;
BEGIN
  x := 012345678H;
  LC.CardToStr( x, s, 16, ok );
  IO.WrStr( 'ok ' );
  IO.WrCard( ORD( ok ), 1 );
  IO.WrStr( ' s "' );
  IO.WrStr( s );
  IO.WrStr( '"' );
  IO.WrLn();
END Test01;

PROCEDURE WrHex( x-: ARRAY OF BYTE );
VAR
  s: ARRAY [0..127] OF CHAR;
  ok: BOOLEAN;
BEGIN
  LC.CardToStr( x, s, 16, ok );
  IF ok THEN
    IO.WrStr( s );
  ELSE
    IO.WrStr( '???' );
  END;
END WrHex;

CONST
  WrLn ::= IO.WrLn;
  WrStr ::= IO.WrStr;

PROCEDURE Test02();
VAR
  x: Types.TLngCard;
  y: Types.TLngCard;
  z: LC.TCard05;
BEGIN
  x := 0FFFFFFFFH;
  y := 0FFFFFFFFH;
  IF ~ LC.Add( x, y, z ) THEN Fatal( 'Test02: Add failed' ) END;
  WrHex( x ); WrStr( ' + ' ); WrHex( y ); WrStr( ' = ' ); WrHex( z ); WrLn();
END Test02;

PROCEDURE Test03();
VAR
  x: Types.TLngCard;
  y: Types.TLngCard;
  z: LC.TCard06;
BEGIN
  x := 0FFFFFFEEH;
  y := 0FFH;
  IF ~ LC.Sub( x, y, z ) THEN Fatal( 'Test03: Sub failed' ) END;
  WrHex( x ); WrStr( ' - ' ); WrHex( y ); WrStr( ' = ' ); WrHex( z ); WrLn();
END Test03;

PROCEDURE Test04();
VAR
  x: Types.TLngCard;
  y: Types.TShtCard;
  z: Types.TShtInt;
BEGIN
  x := 0H;
  y := 1H;
  z := LC.Cmp( x, y );
  WrHex( x );
  IF z > 0 THEN
    WrStr( ' > ' );
  ELSIF z < 0 THEN
    WrStr( ' < ' );
  ELSE
    WrStr( ' = ' );
  END;
  WrHex( y ); WrLn();
END Test04;

PROCEDURE Test05();
VAR
  x: Types.TLngCard;
  z: LC.TCard08;
  n: Types.TShtCard;
BEGIN
  x := 0FFFFFFFFH;
  IF ~ LC.Cpy( x, z ) THEN Fatal( 'Test05: Cpy failed' ) END;
  n := 4;
  IF ~ LC.UpS( z, n ) THEN Fatal( 'Test05: UpS failed' ) END;
  WrHex( x ); WrStr( ' * 100H ^ ' ); WrHex( n ); WrStr( ' = ' ); WrHex( z ); WrLn();
END Test05;

PROCEDURE Test06();
VAR
  x: Types.TLngCard;
  y: Types.TShtCard;
  z: LC.TCard05;
BEGIN
  x := 0FFFFFFFFH;
  y := 0FFH;
  IF ~ LC.Mul0( x, y, z ) THEN Fatal( 'Test06: Mul failed' ) END;
  WrHex( x ); WrStr( ' * ' ); WrHex( y ); WrStr( ' = ' ); WrHex( z ); WrLn();
END Test06;

PROCEDURE Test07();
VAR
  x: Types.TLngCard;
  y: Types.TCard;
  z: LC.TCard06;
BEGIN
  x := 0FFFFFFFFH;
  y := 0FFFFH;
  IF ~ LC.Mul( x, y, z ) THEN Fatal( 'Test07: Mul failed' ) END;
  WrHex( x ); WrStr( ' * ' ); WrHex( y ); WrStr( ' = ' ); WrHex( z ); WrLn();
END Test07;

PROCEDURE Test08();
VAR
  x, y, z: Types.TCard;
BEGIN
  x := 0FFH;
  y := 101H;
  IF ~ LC.Mul( x, y, z ) THEN Fatal( 'Test08: Mul failed' ) END;
  WrHex( x ); WrStr( ' * ' ); WrHex( y ); WrStr( ' = ' ); WrHex( z ); WrLn();
END Test08;

PROCEDURE Test09();
VAR
  x: Types.TLngCard;
  y: Types.TCard;
  z: Types.TLngCard;
BEGIN
  FOR x := 0 TO 10001H DO
    IF x MOD 100H = 0 THEN IO.WrLngHex( x, 1 ); IO.WrLn() END;
    FOR y := 0 TO 0FFFFH DO
      IF ~ LC.Mul( x, y, z ) THEN
        IO.WrStr( 'x ' ); IO.WrLngHex( x, 1 ); IO.WrLn();
        IO.WrStr( 'y ' ); IO.WrLngHex( y, 1 ); IO.WrLn();
        Fatal( 'Test09: Mul failed' );
      END;
      IF z # x * y THEN Fatal( 'Test09: Wrong result' ) END;
    END
  END;
END Test09;

PROCEDURE Test10();
VAR
  x, y, r: Types.TCard;
  z: Types.TShtCard;
BEGIN
  x := 1234H;
  y := 100H;
  IF ~ LC.Div( x, y, z, r ) THEN Fatal( 'Test10: Div failed' ) END;
  WrHex( x ); WrStr( ' DIV ' ); WrHex( y ); WrStr( ' = ' ); WrHex( z ); WrLn();
  WrHex( x ); WrStr( ' MOD ' ); WrHex( y ); WrStr( ' = ' ); WrHex( r ); WrLn();
END Test10;

PROCEDURE Test11();
VAR
  x, y, z, r: Types.TCard;
  t: Tmr0.TTicks;
BEGIN
  IO.WrStr( 'Test11 begin' ); IO.WrLn();
  t := Now();
  FOR x := 0 TO 0FFFH DO
    IF x MOD 100H = 0 THEN IO.WrLngHex( x, 1 ); IO.WrLn() END;
    FOR y := 1 TO 0FFFFH DO
      IF ~ LC.Div( x, y, z, r ) THEN Fatal( 'Test11: Div failed' ) END;
      IF ( z # x DIV y ) OR ( r # x MOD y ) THEN
        IO.WrStr( 'x ' ); IO.WrLngHex( x, 1 ); IO.WrLn();
        IO.WrStr( 'y ' ); IO.WrLngHex( y, 1 ); IO.WrLn();
        IO.WrStr( 'z ' ); IO.WrLngHex( z, 1 ); IO.WrLn();
        IO.WrStr( 'r ' ); IO.WrLngHex( r, 1 ); IO.WrLn();
        Fatal( 'Test11: Wrong result' );
      END;
    END;
  END;
  IO.WrStr( 'Test11: ' ); IO.WrLngCard( Tmr0.Diff( t, Now() ), 1 ); IO.WrStr( 'ms' ); IO.WrLn();
END Test11;

PROCEDURE Test12();
VAR
  x, y: Types.TCard;
  z: Types.TShtCard;
BEGIN
  x := 2467H;
  y := 1234H;
  LC.Div0( x, y, z );
  WrHex( x ); WrStr( ' DIV ' ); WrHex( y ); WrStr( ' = ' ); WrHex( z ); WrLn();
END Test12;

PROCEDURE Test13();
VAR
  x, y, z, r: Types.TCard;
  t: Tmr0.TTicks;
BEGIN
  IO.WrStr( 'Test13: begin' ); IO.WrLn();
  t := Now();
  FOR x := 0 TO 0FFFH DO
    IF x MOD 100H = 0 THEN IO.WrLngHex( x, 1 ); IO.WrLn() END;
    FOR y := 1 TO 0FFFFH DO
      IF ~ LC.DivA( x, y, z, r ) THEN Fatal( 'Test13: DivA failed' ) END;
      IF ( z # x DIV y ) OR ( r # x MOD y ) THEN
        IO.WrStr( 'x ' ); IO.WrLngHex( x, 1 ); IO.WrLn();
        IO.WrStr( 'y ' ); IO.WrLngHex( y, 1 ); IO.WrLn();
        IO.WrStr( 'z ' ); IO.WrLngHex( z, 1 ); IO.WrLn();
        IO.WrStr( 'r ' ); IO.WrLngHex( r, 1 ); IO.WrLn();
        Fatal( 'Test13: Wrong result' );
      END;
    END;
  END;
  IO.WrStr( 'Test13: ' ); IO.WrLngCard( Tmr0.Diff( t, Now() ), 1 ); IO.WrStr( 'ms' ); IO.WrLn();
END Test13;

PROCEDURE Test14();
VAR
  x, y, z, r: Types.TCard;
  t: Tmr0.TTicks;
BEGIN
  IO.WrStr( 'Test14: begin' ); IO.WrLn();
  t := Now();
  FOR x := 0 TO 0FFFH DO
    IF x MOD 100H = 0 THEN IO.WrLngHex( x, 1 ); IO.WrLn() END;
    FOR y := 1 TO 0FFFFH DO
      IF ~ LC.DivB( x, y, z, r ) THEN Fatal( 'Test14: DivB failed' ) END;
      IF ( z # x DIV y ) OR ( r # x MOD y ) THEN
        IO.WrStr( 'x ' ); IO.WrLngHex( x, 1 ); IO.WrLn();
        IO.WrStr( 'y ' ); IO.WrLngHex( y, 1 ); IO.WrLn();
        IO.WrStr( 'z ' ); IO.WrLngHex( z, 1 ); IO.WrLn();
        IO.WrStr( 'r ' ); IO.WrLngHex( r, 1 ); IO.WrLn();
        Fatal( 'Test14: Wrong result' );
      END;
    END;
  END;
  IO.WrStr( 'Test14: ' ); IO.WrLngCard( Tmr0.Diff( t, Now() ), 1 ); IO.WrStr( 'ms' ); IO.WrLn();
END Test14;

PROCEDURE Test15();
VAR
  x, y, z, r: Types.TCard;
  t: Tmr0.TTicks;
BEGIN
  IO.WrStr( 'Test15: begin' ); IO.WrLn();
  t := Now();
  FOR x := 0 TO 0FFFH DO
    IF x MOD 100H = 0 THEN IO.WrLngHex( x, 1 ); IO.WrLn() END;
    FOR y := 1 TO 0FFFFH DO
      IF ~ LC.DivC( x, y, z, r ) THEN Fatal( 'Test15: DivC failed' ) END;
      IF ( z # x DIV y ) OR ( r # x MOD y ) THEN
        IO.WrStr( 'x ' ); IO.WrLngHex( x, 1 ); IO.WrLn();
        IO.WrStr( 'y ' ); IO.WrLngHex( y, 1 ); IO.WrLn();
        IO.WrStr( 'z ' ); IO.WrLngHex( z, 1 ); IO.WrLn();
        IO.WrStr( 'r ' ); IO.WrLngHex( r, 1 ); IO.WrLn();
        Fatal( 'Test15: Wrong result' );
      END;
    END;
  END;
  IO.WrStr( 'Test15: ' ); IO.WrLngCard( Tmr0.Diff( t, Now() ), 1 ); IO.WrStr( 'ms' ); IO.WrLn();
END Test15;

BEGIN
  Test13();
  Test14();
  Test15();
END LrgCards.
